create or replace package body Observer_Pattern_Helper is

  g_hashmap hashmap;

 function key_value_string_to_hashmap
 ( p_string in varchar2 -- semicolon separated list of key=value pairs. For example: a=1;b=2;
 ) return hashmap
 is   
   l_found boolean:=false;
   l_pos_semi number;
   l_pos_equals number;
   l_string varchar2(32700):= p_string;
   l_hashmap hashmap;
   l_key varchar2(500);
   l_value varchar2(4000);
 begin
   -- I have tried to use dbms_utility.comma_to_table but ran into some of its limitations; see http://asktom.oracle.com/pls/ask/f?p=4950:8:9649128002993894306::NO::F4950_P8_DISPLAYID,F4950_P8_CRITERIA:1415803954123
   loop
     l_pos_semi:= instr( l_string, ';');
     if nvl(l_pos_semi,0) = 0
     then
       exit;
     else
       l_pos_equals:= instr( l_string, '=');
       l_key:= substr(l_string, 1, l_pos_equals-1);
       l_value:= substr(l_string, l_pos_equals+1, l_pos_semi  - l_pos_equals -1);
       l_hashmap(l_key ):= l_value;
       l_string:= substr(l_string, l_pos_semi +1);
     end if;
   end loop;
   return l_hashmap;
 end key_value_string_to_hashmap;
 
 procedure set_hashmap
 ( p_hashmap in hashmap
 ) is
 begin
   g_hashmap:= p_hashmap;
 end set_hashmap;
 
 function get_hashmap
 return hashmap
 is
 begin
   return g_hashmap;
 end get_hashmap;
 
  procedure register_observer
  ( p_observer   in     varchar2              
  , p_event_type in     varchar2 default null -- when null, then observe all event types  
  , p_observers  in out observer_table_t
  ) is
    l_observer observer_t;
  begin
  dbms_output.put_line('register '||p_observer||';'||p_event_type);
   l_observer.observer := p_observer;
   l_observer.event_type:= p_event_type;
   p_observers.extend;
   p_observers(p_observers.last):= l_observer;
  end register_observer;  

  procedure notify_observers
  ( p_event_type       in varchar2
  , p_event_parameters in hashmap
  , p_observable       in varchar2
  , p_observers        in observer_table_t
  ) is
    i number;
  begin
    set_hashmap(p_hashmap => p_event_parameters);
    -- notify all registerd observers that are interested in this event_type
    i := p_observers.FIRST;  
    WHILE i IS NOT NULL LOOP
      if nvl(p_observers(i).event_type, p_event_type) = p_event_type
      then
        execute immediate 
         'begin
           '||p_observers(i).observer
           ||' ( p_observable   => :1
               , p_event_type   => :2
               , p_event_values => Observer_Pattern_Helper.get_hashmap
               );
            end;'
         using in p_observable
         ,     in p_event_type
         ;
      end if;
      i := p_observers.NEXT(i);  -- get subscript of next element
    END LOOP;
  end notify_observers;

 
end Observer_Pattern_Helper;
/
