create or replace package body Performance_Monitor is
  -- Author  : LUCAS_J
  -- Created : 3/11/2006 7:51:56 AM
  -- Purpose : This package is a close observer for Performance Issues in our application. It will respond in an appropriate fashion to notifications of performance loss


  g_logger         varchar2(120);
  procedure set_logger (p_logger in varchar2)
  is
  begin
    g_logger := p_logger;
  end set_logger;

  procedure log
  ( p_log_message in varchar2
  , p_log_level   in varchar2 default 'debug'
  , p_log_origin  in varchar2 default 'HRM_SALARY_RULES'
  ) is 
  begin
    if g_logger is not null
    then
      execute immediate 
      'begin
         '||g_logger
         ||' ( p_log_level => :1
             , p_log_origin => :2
             , p_log_message => :3
             );
          end;'
       using in p_log_level
       ,     in p_log_origin
       ,     in p_log_message
       ;
    end if;
  end log; 

  
  procedure performance_loss_observer
  ( p_observable in varchar2 -- the name of the package that sent the event 
  , p_event_type in varchar2  
  , p_event_values in observer_pattern_helper.hashmap
  ) is
    l_key varchar2(200);
  begin
    -- here we handle all performance issue events that we receive notification of
    l_key:= p_event_values.first;
    log
    ( p_log_message => 'Performance issue in '||p_observable||': '||l_key||'='||p_event_values(l_key)
    , p_log_level => 'warning'
    );  
  end performance_loss_observer;

end Performance_Monitor;
/
